<?xml version='1.0' encoding='utf-8' ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:msxsl="urn:schemas-microsoft-com:xslt" extension-element-prefixes="msxsl">
<xsl:output method="html" doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN" version="4.0" encoding="utf-8" indent="yes" />

<!-- Copyright(c) 2001 Intel Corporation -->

    <!-- Use this variable to change whether we are using System or some other namespace -->
    <xsl:variable name="top-level-namespace" select="'System'" />

    <!-- This is set by the user if he/she wants more fine grained control
     of the libraries that are transformed. The ECMA submission is one
	 big XML file with more than one library (BCL, Implementation, etc.).
	 By default this stylesheet transforms all libraries. But if the user
	 wants on one or two, he/she should:

	 - MUST comment out __ALL__
	 - UNCOMMENT others or add the libraries desired to the libraries-to-generate variable by
       using the xsl:value-of for each library, MAKING SURE that each library
       is spelled exactly as it is in the XML and a COMMA follows
    -->
    <!-- What libraries in the current XML to generate code from -->
    <xsl:variable name="libraries-to-generate">
        <!-- Default -->
        <xsl:value-of select="'__ALL__'" />
        <!-- Other Possible Options. Must comment out __ALL__ -->
        <!--<xsl:value-of select="'BCL,'"/>-->
        <!--<xsl:value-of select="'RuntimeInfrastructure,'"/>-->
        <!--<xsl:value-of select="'XML,'"/>-->
        <!--<xsl:value-of select="'Reflection,'"/>-->
        <!--<xsl:value-of select="'Networking,'"/>-->
        <!--<xsl:value-of select="'ExtendedNumerics,'"/>-->
        <!--<xsl:value-of select="'ExtendedArray,'"/>-->
    </xsl:variable>

    <xsl:variable name="gend-exceptions">
        <xsl:for-each select="//Types/Type">
            <xsl:if test="contains(@FullName,'Exception') and (contains($libraries-to-generate,MemberOfLibrary) or ($libraries-to-generate='__ALL__'))">
                <xsl:value-of select="@FullName"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="exceptions-with-empty-constructor">
        <xsl:for-each select="//Types/Type">
            <xsl:if test="contains(@FullName,'Exception')">
                <xsl:for-each select="Members/Member">
                    <xsl:if test="MemberType='Constructor' and contains(MemberSignature[position()=2]/@Value,'()')">
                        <xsl:value-of select="../../@FullName"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <!-- Need to concat the , because there are some types that just extend other types in
         regards to their name...e.g. System.Uri and System.UriBuilder. If do a contains on
         a variable looking for UriBuilder, it may find Uri instead -->
    <xsl:variable name="structs">
        <xsl:for-each select="//Types/Type">
            <xsl:if test="contains(TypeSignature[position()=2]/@Value,' struct ')">
                <xsl:value-of select="concat(@FullName,',')"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="enums">
        <xsl:for-each select="//Types/Type">
            <xsl:if test="contains(TypeSignature[position()=2]/@Value,' enum ')">
                <xsl:value-of select="concat(@FullName,',')"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="types-with-aliases">
        <xsl:value-of select="'System.Byte'"/>
        <xsl:value-of select="'System.Int16'"/>
        <xsl:value-of select="'System.Int32'"/>
        <xsl:value-of select="'System.Int64'"/>
        <xsl:value-of select="'System.SByte'"/>
        <xsl:value-of select="'System.UInt16'"/>
        <xsl:value-of select="'System.UInt32'"/>
        <xsl:value-of select="'System.UInt64'"/>
        <xsl:value-of select="'System.Decimal'"/>
        <xsl:value-of select="'System.Double'"/>
        <xsl:value-of select="'System.Single'"/>
        <xsl:value-of select="'System.Boolean'"/>
        <xsl:value-of select="'System.String'"/>
        <xsl:value-of select="'System.Char'"/>
    </xsl:variable>

    <xsl:template name="reverse">
        <xsl:param name="str" />
        <xsl:if test="$str">
            <xsl:call-template name="reverse">
                <xsl:with-param name="str" select="substring($str,2)" />
            </xsl:call-template>
        </xsl:if>
        <xsl:value-of select="substring($str,1,1)" />
    </xsl:template>
    
    <!-- Extract the namespaces from the ILASM signatures to figure out what should be included in
     the "using directives". Need to recursively call this template to get all the strings in the
     signature-->
    <xsl:template name="extract">
        <xsl:param name="str" />
        <xsl:variable name="strspace" select="concat(normalize-space($str),' ')" />
        <xsl:variable name="first" select="substring-before($strspace,' ')" />
        <xsl:variable name="rest" select="substring-after($strspace,' ')" />
        <xsl:if test="starts-with($first, 'System.')">
            <!-- strip out the class name by reversing -->
            <xsl:variable name="temp">
                <xsl:call-template name="reverse">
                    <xsl:with-param name="str" select="$first" />
                </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="temp2">
                <!-- Remove any of the generics classes from the using, or just the normal class after the last . -->
                <!-- Example case: System.Collections.Generic.Dictionary<K,V>.Enumerator -->
                <!-- Don't want the Dictionary<K,V> as part of the using directive either -->
                <xsl:variable name="temp3">
                    <xsl:choose>
                        <xsl:when test="contains($temp, '&lt;')">
                            <xsl:value-of select="substring-after($temp,'&lt;')" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$temp" />
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:call-template name="reverse">
                    <xsl:with-param name="str" select="substring-after($temp3,'.')" />
                </xsl:call-template>
            </xsl:variable>
            <directive><xsl:value-of select="$temp2"/></directive> 
        </xsl:if>
        <xsl:if test="$rest">
            <xsl:call-template name="extract">
                <xsl:with-param name="str" select="$rest" />
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="/">
        <HTML style="">
            <HEAD style="" />
            <BODY style="">
                <P style="">
                    <xsl:apply-templates/>
                </P>
            </BODY>
        </HTML>
    </xsl:template>
    
    <xsl:template name="InitialData">
        <!-- Place the libraries at the top of the document in order to let the tool know which ones
             are represented in this run. For naming of document purposes -->
        <xsl:value-of select="concat($libraries-to-generate, ',')"/>
        <xsl:value-of select="'#'"/>
    </xsl:template>

    <xsl:template match="*|/">
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="text()|@*">
        <xsl:value-of select="." />
    </xsl:template>
    
    <xsl:template match="Libraries">
        <xsl:call-template name="InitialData"/>
        <xsl:apply-templates select="Types" mode="FromLibrariesTemplate"/>
    </xsl:template>
    
     <xsl:template match="Types" mode="FromLibrariesTemplate">
        <!-- Only select the Type if it is in the selected library -->
        <xsl:if test="$libraries-to-generate='__ALL__' or contains($libraries-to-generate,@Library)">
            <xsl:apply-templates select="Type"/>
        </xsl:if>    
    </xsl:template>

    <!-- This one should only be called if there is only one library in the XML being 
         rendered, thus there is not a "Libraries" template match -->
    <xsl:template match="Types">
        <xsl:call-template name="InitialData"/>    
        <!-- Only select the Type if it is in the selected library -->
        <xsl:if test="$libraries-to-generate='__ALL__' or contains($libraries-to-generate,@Library)">
            <xsl:apply-templates select="Type" />
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="Type">
		<!-- A TypeExcluded = '1' should never happen -->
        <xsl:if test="TypeExcluded='0'">
            <!-- An "interesting" way to get the namespace. This makes sure the last . is removed in case of a class of the form: example: System.IO.Stream -->
            <!-- BUG FIX: 24 Oct 2001....The System.Threading.Thread case where the Name of the class is a substring of one of the namespace words -->
            <xsl:variable name="nsTemp3" select="concat(@FullName, '#')"/>
            <xsl:variable name="nsTemp" select="concat(substring-before($nsTemp3,concat(@Name,'#')),'#')" />
            <xsl:variable name="nsTemp2" select="substring-before($nsTemp,'.#')" />
            <xsl:variable name="nsFinal" select="concat($top-level-namespace,substring-after($nsTemp2,'System'))" />
            <!-- Getting the using directives working -->
            <xsl:variable name="using-directives-with-duplicates">
                <xsl:call-template name="extract">
                    <xsl:with-param name="str" select="TypeSignature[position()=1]/@Value" />
                </xsl:call-template>
                <xsl:for-each select="./Members/Member/ReturnValue">
                    <xsl:if test="not(../Excluded='1' and not( contains($libraries-to-generate,../ExcludedLibrary) or ($libraries-to-generate='__ALL__')))">
                        <xsl:call-template name="extract">
                            <xsl:with-param name="str" select="ReturnType" />
                        </xsl:call-template>
                    </xsl:if>
                </xsl:for-each>
                <xsl:for-each select="./Members/Member/Parameters/Parameter">
                    <xsl:if test="not(../../Excluded='1' and not( contains($libraries-to-generate,../../ExcludedLibrary) or ($libraries-to-generate='__ALL__')))">
                        <xsl:call-template name="extract">
                            <xsl:with-param name="str" select="@Type" />
                        </xsl:call-template>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="using-directives-no-duplicates">
                <xsl:for-each select="msxsl:node-set($using-directives-with-duplicates)/directive">
                    <xsl:if test="not(.=following::directive)">
                        <directive><xsl:value-of select="concat($top-level-namespace,substring-after(.,'System'))"/></directive>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <pre>// The signatures for this code were automatically generated from the 
// standard ECMA CLI XML Library Specification. This class/interface/enum 
// is a part of the ECMA CLI standard library specification.
/*---------------------------------------------------------------------
CREATED: October 05, 2001
MODIFIED: February 05, 2010
OWNER: code_gen
VERSION: 0.1
---------------------------------------------------------------------*/
            </pre>
            <pre>
                <xsl:value-of select="concat('using ',$top-level-namespace, ';&#xA;')"/>
                <xsl:for-each select="msxsl:node-set($using-directives-no-duplicates)/directive">
                    <!-- I always put a using System (or top level namespace in the code -->
                    <xsl:if test="not(.=$top-level-namespace)">
                        <xsl:value-of select="concat('using ',., ';&#xA;')"/>    
                    </xsl:if>
                </xsl:for-each>
            </pre>
            <pre>
                <xsl:value-of select="'&#xA;'" />
            </pre>
            <pre>
                <xsl:value-of select="concat('namespace ',$nsFinal)" />
            </pre>
            <pre>
                <xsl:value-of select="'{'" />
            </pre>
            <xsl:apply-templates select="Attributes" mode="Type"/>
            <xsl:apply-templates select="Docs" mode="TypeSummary" />
            <xsl:apply-templates select="TypeSignature" />
            <!-- Next set of info exists only if we are not dealing with a delegate -->
            <xsl:variable name="type-classification" select="./TypeSignature[position()=2]/@Value" />
            <xsl:if test="not(contains($type-classification,'delegate'))">
                <pre>
                    <xsl:value-of select="'    {'" />
                </pre>
                <xsl:apply-templates select="Members" />
                <pre>
                    <xsl:value-of select="'    }'" />
                </pre>
            </xsl:if>
            <pre>
                <xsl:value-of select="'}#TYPE BREAK#'" />
            </pre>
            <!--<pre>
                <xsl:value-of select="'#TYPE BREAK#'" />
            </pre>-->
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="Members">
        <!-- used to determine what the type classification is for the given
		 type (e.g. struct, class, interface). Need the position function
		 so to access the C# version of the signature....there are two
		 TypeSignature element tags, remember.

		 SHOULD THERE BE A TYPE CLASSIFICATION TAG IN THE XML?-->
        <xsl:variable name="type-classification" select="../TypeSignature[position()=2]/@Value" />
        <xsl:choose>
            <xsl:when test="contains($type-classification,'struct')">
                <xsl:apply-templates select="Member" mode="class">
                    <xsl:sort select="MemberType" order="ascending" />
                    <xsl:sort select="@MemberName" order="ascending" />
                </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="contains($type-classification,'interface')">
                <xsl:apply-templates select="Member" mode="interface">
                    <xsl:sort select="MemberType" order="ascending" />
                    <xsl:sort select="@MemberName" order="ascending" />
                </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="contains($type-classification,'class')">
                <xsl:apply-templates select="Member" mode="class">
                    <xsl:sort select="MemberType" order="ascending" />
                    <xsl:sort select="@MemberName" order="ascending" />
                </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="contains($type-classification,'enum')">
                <xsl:apply-templates select="Member" mode="enum">
                    <xsl:sort select="MemberType" order="ascending" />
                    <xsl:sort select="@MemberName" order="ascending" />
                </xsl:apply-templates>
            </xsl:when>
            <!--<xsl:when test="contains($type-classification,'delegate')">
                //#### Is there anything that needs to be done for delegates ####
            </xsl:when>-->
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="Member" mode="interface">
        <xsl:apply-templates select="Docs" mode="MemberSummary" />
        <xsl:apply-templates select="Docs" mode="MemberParams" />
        <xsl:apply-templates select="Docs" mode="MemberReturns" />
        <xsl:apply-templates select="Docs" mode="MemberExceptions" />
        <xsl:apply-templates select="MemberSignature" mode="interface" />
        <xsl:choose>
            <xsl:when test="not(position() = last())">
                <pre>
                    <xsl:value-of select="'&#xA;&#xD;'" />
                </pre>
            </xsl:when>
            <xsl:otherwise>
                <pre>
                    <xsl:value-of select="''" />
                </pre>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="Member" mode="class">
        <xsl:variable name="ebname" select="ExcludedLibrary" />
        <xsl:choose>
            <!-- Comment out member if excluded and not its library not part of current code gen-->
            <xsl:when test="Excluded='1' and not( contains($libraries-to-generate,$ebname) or ($libraries-to-generate='__ALL__'))">
                <pre>
                    <xsl:value-of select="'        /*'" />
                </pre>
                <xsl:choose>
                    <xsl:when test="$ebname='Implementation'">
                        <pre>
                            <xsl:value-of select="'        ### THE FOLLOWING MUST BE IMPLEMENTED IF IMPLEMENTING THE RuntimeInfrastructure LIBRARY ###'" />
                        </pre>
                    </xsl:when>
                    <xsl:when test="$ebname='ExtendedArray'">
                        <pre>
                            <xsl:value-of select="'        ### THE FOLLOWING MUST BE IMPLEMENTED IF YOUR SYSTEM SUPPORTS Extended Arrays ###'" />
                        </pre>
                    </xsl:when>
                    <xsl:otherwise>
                        <pre>
                            <xsl:value-of select="concat('        ### THE FOLLOWING MUST BE IMPLEMENTED IF IMPLEMENTING THE ',$ebname,' LIBRARY ###')" />
                        </pre>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:apply-templates select="Attributes" mode="Member"/>
                <xsl:apply-templates select="Docs" mode="MemberSummary" />
                <xsl:apply-templates select="Docs" mode="MemberParams" />
                <xsl:apply-templates select="Docs" mode="MemberReturns" />
                <xsl:apply-templates select="Docs" mode="MemberExceptions" />
                <xsl:apply-templates select="MemberSignature" mode="class" />
                <pre>
                    <xsl:value-of select="'        */'" />
                </pre>
            </xsl:when>
            <!-- Library member must be part of current code gen, but let user know -->
            <xsl:when test="Excluded='1'">
                <xsl:choose>
                    <xsl:when test="$ebname='Implementation'">
                        <pre>
                            <xsl:value-of select="'        //THE FOLLOWING IS NOT COMMENTED OUT BECAUSE THE RuntimeInfrastructure LIBRARY WAS PART OF CODE GEN'" />
                        </pre>
                    </xsl:when>
                    <xsl:otherwise>
                        <pre>
                            <xsl:value-of select="concat('        //THE FOLLOWING IS NOT COMMENTED OUT BECAUSE THE ',$ebname,' LIBRARY WAS PART OF CODE GEN')" />
                        </pre>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:apply-templates select="Attributes" mode="Member"/>
                <xsl:apply-templates select="Docs" mode="MemberSummary" />
                <xsl:apply-templates select="Docs" mode="MemberParams" />
                <xsl:apply-templates select="Docs" mode="MemberReturns" />
                <xsl:apply-templates select="Docs" mode="MemberExceptions" />
                <xsl:apply-templates select="MemberSignature" mode="class" />
            </xsl:when>
            <!-- Not exlcuded -->
            <xsl:otherwise>
                <xsl:apply-templates select="Attributes" mode="Member"/>
                <xsl:apply-templates select="Docs" mode="MemberSummary" />
                <xsl:apply-templates select="Docs" mode="MemberParams" />
                <xsl:apply-templates select="Docs" mode="MemberReturns" />
                <xsl:apply-templates select="Docs" mode="MemberExceptions" />
                <xsl:apply-templates select="MemberSignature" mode="class" />
            </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="not(position() = last())">
                <pre>
                    <xsl:value-of select="'&#xA;&#xD;'" />
                </pre>
            </xsl:when>
            <xsl:otherwise>
                <pre>
                    <xsl:value-of select="''" />
                </pre>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
   
    
    <xsl:template match="Member" mode="enum">
        <xsl:if test="not(normalize-space(@MemberName)='value__')">
             <xsl:apply-templates select="Attributes" mode="Member"/>
            <xsl:apply-templates select="MemberSignature" mode="enum"/>
        </xsl:if>  
    </xsl:template>
    
    <xsl:template match="MemberSignature" mode="enum">
        <xsl:if test="@Language='C#'">
            <xsl:variable name="ebname" select="../ExcludedLibrary"/>
            <xsl:choose>
                <xsl:when test="../Excluded='1' and not( contains($libraries-to-generate,$ebname) or ($libraries-to-generate='__ALL__'))">
                    <pre><xsl:value-of select="concat('        //### THE FOLLOWING MUST BE UNCOMMENTED IF IMPLEMENTING THE ',$ebname,' LIBRARY ###')"/></pre>
                    <pre><xsl:value-of select="concat('        //',translate(@Value,';',','))"/></pre>
                </xsl:when>
                <xsl:otherwise>
                    <pre><xsl:value-of select="concat('        ',translate(@Value,';',','))"/></pre>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>


    <xsl:template match="TypeSignature">
        <xsl:if test="@Language='C#'">
            <font face="Courier New">
				<!-- In case there is an ExcludedBaseTypeName. For example, in the case
			         of Type, where it can inherit from Object or MemberInfo -->
                <xsl:choose>
                    <xsl:when test="../Base/ExcludedBaseTypeName">
                        <xsl:choose>
                            <xsl:when test="contains($libraries-to-generate,../Base/ExcludedLibraryName) or ($libraries-to-generate='__ALL__')">
                                <xsl:variable name="temp">
                                    <xsl:call-template name="reverse">
                                        <xsl:with-param name="str" select="../Base/ExcludedBaseTypeName" />
                                    </xsl:call-template>
                                </xsl:variable>
                                <xsl:variable name="temp2">
                                    <xsl:call-template name="reverse">
                                        <xsl:with-param name="str" select="substring-before($temp,'.')" />
                                    </xsl:call-template>
                                </xsl:variable>
                                <pre><xsl:value-of select="concat('    ',substring-before(@Value,':'),' : ',$temp2)"/></pre>
                            </xsl:when>
                            <xsl:otherwise>
                                <pre>
                                    <xsl:value-of select="concat('    ',@Value)" />
                                </pre>    
                            </xsl:otherwise>
                        </xsl:choose>  
                    </xsl:when>
                    <xsl:otherwise>
                        <pre>
                            <xsl:value-of select="concat('    ',@Value)" />
                        </pre>
                    </xsl:otherwise>
                </xsl:choose>
            </font>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="Docs" mode="TypeSummary">
        <xsl:apply-templates select="summary" mode="Type" />
    </xsl:template>
    
    <xsl:template match="Docs" mode="MemberSummary">
        <xsl:apply-templates select="summary" />
    </xsl:template>
    
    <xsl:template match="Docs" mode="MemberParams">
        <xsl:apply-templates select="param" />
    </xsl:template>
    
    <xsl:template match="Docs" mode="MemberReturns">
        <!-- For methods -->
        <xsl:if test="returns">
            <xsl:apply-templates select="returns" />
        </xsl:if>
        <!-- For properties -->
        <xsl:if test="value">
            <xsl:apply-templates select="value" />
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="Docs" mode="MemberExceptions">
        <xsl:apply-templates select="exception" mode="comments" />
    </xsl:template>
    
    <xsl:template match="summary">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="'        ///&lt;summary&gt;'" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/summary&gt;'" />
        </pre>
    </xsl:template>
    
    <!-- 4 less spaces of indention for type summaries -->
    <xsl:template match="summary" mode="Type">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="'    ///&lt;summary&gt;'" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/summary&gt;'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="param">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="concat('        ///&lt;param name=&quot;',@name,'&quot;&gt;')" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/param&gt;'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="returns">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="'        ///&lt;returns&gt;'" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/returns&gt;'" />
        </pre>
    </xsl:template>

    <xsl:template match="value">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="'        ///&lt;returns&gt;'" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/returns&gt;'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="exception" mode="comments">
        <xsl:variable name="t">
            <xsl:apply-templates/>
        </xsl:variable>
        <xsl:variable name="w" select="translate($t,'&#xA;&#xD;','')" />
        <pre>
            <xsl:value-of select="concat('        ///&lt;exception cref=&quot;',substring-after(@cref, ':'),'&quot;&gt;')" />
            <xsl:value-of select="$w" />
            <xsl:value-of select="'&lt;/exception&gt;'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="MemberSignature" mode="interface">
        <xsl:if test="@Language='C#'">
            <font face="Courier New">
                <xsl:choose>
                    <xsl:when test="../MemberType='Method'">
                        <pre>
                            <xsl:value-of select="concat('        ',@Value)" />
                        </pre>
                    </xsl:when>
                    <xsl:when test="../MemberType='Property'">
                        <xsl:variable name="sig">
                            <xsl:value-of select="@Value" />
                        </xsl:variable>
                        <pre>
                            <xsl:value-of select="concat('        ',substring-before($sig, '{'))" />
                        </pre>
                        <pre>
                            <xsl:value-of select="'        {'" />
                        </pre>
                        <xsl:if test="contains($sig,'get;')">
                            <pre>
                                <xsl:value-of select="'            get;'" />
                            </pre>
                        </xsl:if>
                        <xsl:if test="contains($sig,'set;')">
                            <pre>
                                <xsl:value-of select="'            set;'" />
                            </pre>
                        </xsl:if>
                        <pre>
                            <xsl:value-of select="'        }'" />
                        </pre>
                    </xsl:when>
                    <xsl:when test="../MemberType='Field'">
                        <pre><xsl:value-of select="concat('        ',@Value)" /></pre>
                    </xsl:when>
                </xsl:choose>
            </font>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="AbstractClassMethod">
        <xsl:param name="sig" />
        <pre>
            <xsl:value-of select="concat('        ',$sig)" />
        </pre>
    </xsl:template>
    
    <xsl:template name="AbstractClassProperty">
        <xsl:param name="sig" />
        <pre>
            <xsl:value-of select="concat('        ',substring-before($sig, '{'))" />
        </pre>
        <pre>
            <xsl:value-of select="'        {'" />
        </pre>
        <xsl:if test="contains($sig,'get;')">
            <pre>
                <xsl:value-of select="'            get;'" />
            </pre>
        </xsl:if>
        <xsl:if test="contains($sig,'set;')">
            <pre>
                <xsl:value-of select="'            set;'" />
            </pre>
        </xsl:if>
        <pre>
            <xsl:value-of select="'        }'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="MemberSignature" mode="class">
        <xsl:if test="@Language='C#'">
            <font face="Courier New">
                <xsl:choose>
                    <xsl:when test="../MemberType='Method' or ../MemberType='Constructor'">
                        <xsl:choose>
                            <xsl:when test="contains(@Value,'abstract')">
                                <xsl:call-template name="AbstractClassMethod">
                                    <xsl:with-param name="sig" select="@Value" />
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <!-- Remove the ; at the end of the signature -->
                                <pre>
                                    <xsl:value-of select="concat('        ',substring-before(@Value, ';'))" />
                                </pre>
                                <pre>
                                    <xsl:value-of select="'        {'" />
                                </pre>
                                <pre>
                                    <!-- &quot; is the representation for " when needed in a string literal -->
                                    <xsl:value-of select="concat('            ','System.Console.WriteLine(&quot;',../../../@Name,': ',../@MemberName,'(')" />
                                    <xsl:apply-templates select="../Parameters" mode="code" />
                                    <xsl:value-of select="'): Stub Method&quot;);'" />
                                </pre>
                                <xsl:apply-templates select="../Docs" mode="Exception" />
                                <xsl:apply-templates select="../ReturnValue" />
                                <pre>
                                    <xsl:value-of select="'        }'" />
                                </pre>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="../MemberType='Property'">
                        <xsl:choose>
                            <xsl:when test="contains(@Value,'abstract')">
                                <xsl:call-template name="AbstractClassProperty">
                                    <xsl:with-param name="sig" select="@Value" />
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:variable name="sig" select="@Value" />
                                <pre>                                  
                                    <xsl:value-of select="concat('        ',substring-before($sig, '{'))" />
                                </pre>
                                <pre>
                                    <xsl:value-of select="'        {'" />
                                </pre>
                                <xsl:if test="contains($sig,'get;')">
                                    <pre>
                                        <xsl:value-of select="'            get'" />
                                    </pre>
                                    <pre>
                                        <xsl:value-of select="'            {'" />
                                    </pre>
                                    <pre>
                                        <xsl:value-of select="concat('                ','System.Console.WriteLine(&quot;',../../../@Name,'_',../@MemberName,': Stub Property&quot;);')" />
                                    </pre>
                                    <xsl:apply-templates select="../Docs" mode="ExceptionProp" />
                                    <xsl:apply-templates select="../ReturnValue" mode="Property" />
                                    <pre>
                                        <xsl:value-of select="'            }'" />
                                    </pre>
                                </xsl:if>
                                <xsl:if test="contains($sig,'set;')">
                                    <pre>
                                        <xsl:value-of select="'            set'" />
                                    </pre>
                                    <pre>
                                        <xsl:value-of select="'            {'" />
                                    </pre>
                                    <pre>
                                        <xsl:value-of select="concat('                ','System.Console.WriteLine(&quot;',../../../@Name,': ',../@MemberName,'(')" />
                                        <xsl:apply-templates select="../Parameters" mode="code" />
                                        <xsl:value-of select="'): Stub Property&quot;);'" />
                                    </pre>
                                    <xsl:apply-templates select="../Docs" mode="ExceptionProp" />
                                    <pre>
                                        <xsl:value-of select="'            }'" />
                                    </pre>
                                </xsl:if>
                                <pre>
                                    <xsl:value-of select="'        }'" />
                                </pre>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="../MemberType='Field'">
                        <pre><xsl:value-of select="concat('        ',@Value)" /></pre>
                    </xsl:when>
                </xsl:choose>
            </font>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="Docs" mode="Exception">
        <xsl:apply-templates select="exception" mode="code" />
    </xsl:template>
    
    <xsl:template match="Docs" mode="ExceptionProp">
        <xsl:apply-templates select="exception" mode="codeProp" />
    </xsl:template>
    
    <xsl:template match="ReturnValue">
        <!-- See if there are any out parameters. If so, go through the motions -->
        <xsl:variable name="memSig" select="../MemberSignature[position()=2]/@Value"/>
        <xsl:if test="contains($memSig,', out ')">
            <xsl:variable name="subMemSig" select="concat('out ', substring-before(substring-after($memSig, ', out '),')'), ',')"/>
            <xsl:call-template name="findOutParams">
                <xsl:with-param name="sig" select="$subMemSig"/>
            </xsl:call-template>
        </xsl:if>
      
        <xsl:if test="ReturnType and not(ReturnType='System.Void')">          
            <xsl:variable name="rt" select="ReturnType"/>
            <xsl:choose>
                <!-- Get rid of the arrays first -->
                <xsl:when test="contains($rt,'[]')">
                    <pre>
                        <xsl:value-of select="concat('            ','return null;')" />
                    </pre>
                </xsl:when>
                <xsl:when test="contains($types-with-aliases, $rt)">
                    <xsl:choose>
                        <xsl:when test="contains($rt, 'System.Char')">
                            <pre>
                                <xsl:value-of select="concat('            ','return (char) 0;')" />
                            </pre>
                        </xsl:when>
                        <xsl:when test="contains($rt, 'System.String')">
                            <pre>
                                <xsl:value-of select="concat('            ','return null;')" />
                            </pre>
                        </xsl:when>
                        <xsl:when test="contains($rt, 'System.Boolean')">
                            <pre>
                                <xsl:value-of select="concat('            ','return false;')" />
                            </pre>
                        </xsl:when>
                        <xsl:otherwise>
                            <pre>
                                <xsl:value-of select="concat('            ','return 0;')" />
                            </pre>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="contains($enums,concat($rt,','))">
                    <pre>
                        <xsl:value-of select="concat('            ','return 0;')" />
                    </pre>
                </xsl:when>
                <xsl:when test="contains($structs,concat($rt,','))">
                    <xsl:variable name="temp">
                        <xsl:call-template name="reverse">
                            <xsl:with-param name="str" select="$rt" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="temp2">
                        <xsl:call-template name="reverse">
                            <xsl:with-param name="str" select="substring-before($temp,'.')" />
                        </xsl:call-template>
                    </xsl:variable>
                    <pre>
                        <xsl:value-of select="concat('            ','return new ',$temp2,'();')" />
                    </pre>    
                </xsl:when>
                <xsl:otherwise>
                    <pre>
                        <xsl:value-of select="concat('            ','return null;')" />
                    </pre>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>
    
    <!-- Recursively goes through a member sub-signature and finds all of the out parameters and print out code that sets these out parameters to something-->
    <xsl:template name="findOutParams">
        <xsl:param name="sig"/>
        <xsl:variable name="thisParam" select="substring-before($sig, ',')"/>
        <xsl:variable name="restSig" select="substring-after($sig, ',')"/>
        <xsl:if test="restSig">
            <xsl:call-template name="findOutParams">
                <xsl:with-param name="sig" select="$restSig"/>
            </xsl:call-template>
        </xsl:if>
        <!-- Make sure we are at an out parameter and not some freaky thing where the variable is named myOut or something -->
        <xsl:if test="substring($thisParam, 1, 4)='out '">
            <!-- An out param looks like: out int aParam.....get the 'aParam'-->
            <xsl:variable name="noOut" select="substring-after($thisParam, 'out ')"/>
            <xsl:variable name="paramName" select="substring-after($noOut, ' ')"/>
            <pre>
                <xsl:value-of select="concat('            ',$paramName,' = 0;')" />
            </pre>    
        </xsl:if>
    </xsl:template>


    <!-- Need an extra 4 spaces for properties -->
    <xsl:template match="ReturnValue" mode="Property">
        <xsl:if test="ReturnType and not(ReturnType='System.Void')">
            <xsl:variable name="rt" select="ReturnType"/>
            <xsl:choose>
                <!-- Get rid of the arrays first -->
                <xsl:when test="contains($rt,'[]')">
                    <pre>
                        <xsl:value-of select="concat('                ','return null;')" />
                    </pre>
                </xsl:when>
                <xsl:when test="contains($types-with-aliases, $rt)">
                    <xsl:choose>
                        <xsl:when test="contains($rt, 'System.Char')">
                            <pre>
                                <xsl:value-of select="concat('                ','return (char) 0;')" />
                            </pre>
                        </xsl:when>
                        <xsl:when test="contains($rt, 'System.String')">
                            <pre>
                                <xsl:value-of select="concat('                ','return null;')" />
                            </pre>
                        </xsl:when>
                        <xsl:when test="contains($rt, 'System.Boolean')">
                            <pre>
                                <xsl:value-of select="concat('                ','return false;')" />
                            </pre>
                        </xsl:when>
                        <xsl:otherwise>
                            <pre>
                                <xsl:value-of select="concat('                ','return 0;')" />
                            </pre>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="contains($enums,concat($rt,','))">
                    <pre>
                        <xsl:value-of select="concat('                ','return 0;')" />
                    </pre>
                </xsl:when>
                <xsl:when test="contains($structs,concat($rt,','))">
                    <xsl:variable name="temp">
                        <xsl:call-template name="reverse">
                            <xsl:with-param name="str" select="$rt" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="temp2">
                        <xsl:call-template name="reverse">
                            <xsl:with-param name="str" select="substring-before($temp,'.')" />
                        </xsl:call-template>
                    </xsl:variable>
                    <pre>
                        <xsl:value-of select="concat('                ','return new ',$temp2,'();')" />
                    </pre>    
                </xsl:when>
                <xsl:otherwise>
                    <pre>
                        <xsl:value-of select="concat('                ','return null;')" />
                    </pre>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>
    
    <!-- for inserting into the Diagnostic call for stub methods/properties -->
    <xsl:template match="Parameters" mode="code">
        <xsl:apply-templates select="Parameter" mode="code" />
    </xsl:template>
    
    <xsl:template match="Parameter" mode="code">
        <!-- Don't put a comma at the last parameter type -->
        <xsl:choose>
            <xsl:when test="not(position() = last())">
                <xsl:value-of select="concat(@Type, ',')" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="@Type" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="exception" mode="code">
        <xsl:variable name="exception-name" select="substring-after(@cref,':')"/>
        <pre>
            <xsl:value-of select="'            if (false)'" />
        </pre>
        <pre>
            <xsl:value-of select="'            {'" />
        </pre>
        <xsl:choose>
            <xsl:when test="not(contains($gend-exceptions, $exception-name))">
                <pre>
                    <xsl:value-of select="concat('                // ',$top-level-namespace,'.',substring-after(@cref, '.'),' was not generated during code gen because its containing library was not selected&#xA;&#xD;')"/>
                    <xsl:value-of select="concat('                throw new ',$top-level-namespace,'.','Exception(); //')" />
                    <xsl:apply-templates/>
                </pre>
            </xsl:when>
            <xsl:when test="not(contains($exceptions-with-empty-constructor, $exception-name))">
                 <pre>
                    <xsl:value-of select="concat('                // ',$top-level-namespace,'.',substring-after(@cref, '.'),' does not have an empty constructor&#xA;&#xD;')"/>
                    <xsl:value-of select="concat('                throw new ',$top-level-namespace,'.','Exception(); //')" />
                    <xsl:apply-templates/>
                </pre>       
            </xsl:when>
            <xsl:otherwise>
                <pre>
                    <xsl:value-of select="concat('                throw new ',$top-level-namespace,'.',substring-after($exception-name,'.'),'(); // ')" />
                    <xsl:apply-templates/>
                </pre>
            </xsl:otherwise>
        </xsl:choose>
        <pre>
            <xsl:value-of select="'            }'" />
        </pre>
    </xsl:template>
    
    <!-- Need 4 more spaces for properties -->
    <xsl:template match="exception" mode="codeProp">
        <xsl:variable name="exception-name" select="substring-after(@cref,':')"/>
        <pre>
            <xsl:value-of select="'                if (false)'" />
        </pre>
        <pre>
            <xsl:value-of select="'                {'" />
        </pre>
        <xsl:choose>
            <xsl:when test="not(contains($gend-exceptions, $exception-name))">
                <pre>
                    <xsl:value-of select="concat('                    // ',$top-level-namespace,'.',substring-after(@cref, '.'),' was not generated during code gen because its containing library was not selected&#xA;&#xD;')"/>
                    <xsl:value-of select="concat('                    throw new ',$top-level-namespace,'.','Exception(); //')" />
                    <xsl:apply-templates/>
                </pre>
            </xsl:when>
            <xsl:when test="not(contains($exceptions-with-empty-constructor, $exception-name))">
                 <pre>
                    <xsl:value-of select="concat('                    // ',$top-level-namespace,'.',substring-after(@cref, '.'),' does not have an empty constructor&#xA;&#xD;')"/>
                    <xsl:value-of select="concat('                    throw new ',$top-level-namespace,'.','Exception(); //')" />
                    <xsl:apply-templates/>
                </pre>       
            </xsl:when>
            <xsl:otherwise>
                <pre>
                    <xsl:value-of select="concat('                    throw new ',$top-level-namespace,'.',substring-after($exception-name,'.'),'(); // ')" />
                    <xsl:apply-templates/>
                </pre>
            </xsl:otherwise>
        </xsl:choose>
        <pre>
            <xsl:value-of select="'                }'" />
        </pre>
    </xsl:template>
    
    <xsl:template match="see">
        <xsl:choose>
            <xsl:when test="@cref">
                <!--Remove the T:, M:, P:, etc. from the string -->
                <xsl:value-of select="substring-after(@cref, ':')" />
            </xsl:when>
            <xsl:when test="@langword">
                <xsl:value-of select="@langword" />
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="paramref">
        <xsl:value-of select="@name" />
    </xsl:template>

    <xsl:template match="Attributes" mode="Type">
        <xsl:apply-templates select="Attribute" mode="Type"/>
    </xsl:template>

    <xsl:template match="Attributes" mode="Member">
        <xsl:apply-templates select="Attribute" mode="Member"/>
    </xsl:template>

    <xsl:template match="Attribute" mode="Type">
        <xsl:if test="not(Excluded='1')">
            <xsl:variable name="attrName" select="AttributeName"/>
            <xsl:choose>
                <xsl:when test="contains($attrName, 'AttributeUsageAttribute')">
                    <pre>
                        <xsl:value-of select="concat('    [AttributeUsage', substring-after($attrName, 'AttributeUsageAttribute'), ']')"/>
                    </pre>
                </xsl:when>
                <xsl:otherwise>
                    <pre>
                        <xsl:value-of select="concat('    [', substring-before($attrName, 'Attribute'), substring-after($attrName, 'Attribute'), ']')"/>
                    </pre>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template match="Attribute" mode="Member">
        <xsl:if test="not(Excluded='1')">
            <xsl:variable name="attrName" select="AttributeName"/>
            <xsl:choose>
                <xsl:when test="contains($attrName, 'AttributeUsageAttribute')">
                    <pre>
                        <xsl:value-of select="concat('    [AttributeUsage', substring-after($attrName, 'AttributeUsageAttribute'), ']')"/>
                    </pre>
                </xsl:when>
                <xsl:otherwise>
                    <pre>
                        <xsl:value-of select="concat('        [', substring-before($attrName, 'Attribute'), substring-after($attrName, 'Attribute'), ']')"/>
                    </pre>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

</xsl:stylesheet>
<!-- Stylus Studio meta-information - (c)1998-2001 eXcelon Corp.
<metaInformation>
<scenarios/>
</metaInformation>
-->
