<?xml version='1.0' encoding='utf-8' ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                xmlns:local="#local-functions"
                version="1.0">				
<xsl:output method="html" 
			doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN" 
			version="4.0" 
			encoding="UTF-8" 
			indent="yes"/>
<!-- Copyright(c) 2001 Intel Corporation -->

<!-- This is set by the user if he/she wants more fine grained control
     of the libraries that are transformed. The ECMA submission is one
	 big XML file with more than one library (BCL, Implementation, etc.).
	 By default this stylesheet transforms all libraries. But if the user
	 wants on one or two, he/she should:

	 - MUST comment out __ALL__
	 - UNCOMMENT others or add the libraries desired to the libraries-to-generate variable by
       using the xsl:value-of for each library, MAKING SURE that each library
       is spelled exactly as it is in the XML and a COMMA follows
-->
<!-- What libraries in the current XML to generate code from -->
<xsl:variable name="libraries-to-generate">
    <!-- Default -->
    <xsl:value-of select="'__ALL__'" />
    <!-- Other Possible Options. Must comment out __ALL__ -->
		<!--<xsl:value-of select="'BCL,'"/>-->
		<!--<xsl:value-of select="'ExtendedNumerics,'"/>-->
		<!--<xsl:value-of select="'Networking,'"/>-->
		<!--<xsl:value-of select="'Parallel,'"/>-->
		<!--<xsl:value-of select="'Reflection,'"/>-->
	    <!--<xsl:value-of select="'RuntimeInfrastructure,'"/>-->
		<!--<xsl:value-of select="'Vararg,'"/>-->
		<!--<xsl:value-of select="'XML,'"/>-->
</xsl:variable>


<xsl:template match="/">
	<HTML style="">
		<HEAD style="">
			<!-- Joel Marcey 24 June 2010 -->
			<!-- Make sure we are encoding in UTF-8 to avoid funky characters like Â before math symbols for example-->
			<!-- See http://stackoverflow.com/questions/1461907/html-encoding-issues-a-character-showing-up-instead-of-nbsp -->
			<!--<xsl:value-of select="'&lt;meta http-equiv=&quot;Content-Type&quot; content=&quot;text/html;charset=utf-8&quot;/&gt;'" />-->
			<meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
			<title>Ecma Class Libraries</title>
		</HEAD>
		<BODY style=""> <!--"font-family:Verdana;font-size:smaller">-->
			<P style="">
				<xsl:apply-templates/>
			</P>
		</BODY>
	</HTML>
</xsl:template>

<xsl:template name="InitialData">
		<!-- Place the libraries at the top of the document in order to let the tool know which ones
             are represented in this run. For naming of document purposes -->
        <xsl:value-of select="concat($libraries-to-generate, ',')"/>
        <xsl:value-of select="'#'"/>
    </xsl:template>

<!-- Joel Marcey 24 June 2010. This prevented the match="/" from being called, which we wanted all along, I think-->
<!--<xsl:template match="*|/">
	<xsl:apply-templates/>
</xsl:template>-->

<xsl:template match="text()|@*">
	<xsl:value-of select="."/>
</xsl:template>

<xsl:template match="Libraries">
        <xsl:call-template name="InitialData"/>
        <xsl:apply-templates select="Types" mode="FromLibrariesTemplate"/>
    </xsl:template>
    
     <xsl:template match="Types" mode="FromLibrariesTemplate">
        <!-- Only select the Type if it is in the selected library -->
        <xsl:if test="$libraries-to-generate='__ALL__' or contains($libraries-to-generate,@Library)">
            <xsl:apply-templates select="Type"/>
        </xsl:if>    
    </xsl:template>

    <!-- This one should only be called if there is only one library in the XML being 
         rendered, thus there is not a "Libraries" template match -->
    <xsl:template match="Types">
        <xsl:call-template name="InitialData"/>    
        <!-- Only select the Type if it is in the selected library -->
        <xsl:if test="$libraries-to-generate='__ALL__' or contains($libraries-to-generate,@Library)">
            <xsl:apply-templates select="Type" />
        </xsl:if>
    </xsl:template>


<xsl:template match="Type">
 	<xsl:if test="TypeExcluded='0'">
		<!-- used to determine what the type classification is for the given
			 type (e.g. struct, class, interface). Need the position function
			 so to access the C# version of the signature....there are two
			 TypeSignature element tags, remember.

			 SHOULD THERE BE A TYPE CLASSIFICATION TAG IN THE XML?
		-->
		<xsl:variable name="type-classification" select="./TypeSignature[position()=2]/@Value"/>
		<font face="Verdana" size="5">
			<b>
				<xsl:value-of select="@FullName"/>
				<xsl:choose>
					<xsl:when test="contains($type-classification,'struct')">
						Structure
					</xsl:when>
					<xsl:when test="contains($type-classification,'interface')">
						Interface
					</xsl:when>
					<xsl:when test="contains($type-classification,'class')">
						Class
					</xsl:when>
					<xsl:when test="contains($type-classification,'enum')">
						Enum
					</xsl:when>
					<xsl:when test="contains($type-classification,'delegate')">
						Delegate
					</xsl:when>
				</xsl:choose>
			</b>
		</font><br/>
		<xsl:apply-templates select="TypeSignature"/>
		<xsl:apply-templates select="AssemblyInfo"/>
		<xsl:apply-templates select="Attributes"/>
		<xsl:apply-templates select="Interfaces"/>
		<xsl:apply-templates select="Docs" mode="ClassSummary"/>
		<xsl:apply-templates select="Docs" mode="ClassParameters"/>
		<xsl:apply-templates select="Base"/>
		<font face="Verdana" size="2">
			<b>Library: </b>
			<xsl:value-of select="MemberOfLibrary"/>
		</font><br/><br/>
		
		<xsl:apply-templates select="ThreadingSafetyStatement"/>
                <!-- JM 27 Dec 2004 -->
                <!-- Added a class permissions template (sparked by MethodInfo) -->
                <xsl:apply-templates select="Docs" mode="ClassPermissions"/>
                <!-- JM 21 Nov 2004 -->
                <!-- Added for delegates that return non-void -->
		<xsl:apply-templates select="Docs" mode="ClassReturns"/>
		<xsl:apply-templates select="Docs" mode="ClassDescription"/>
		<xsl:apply-templates select="Docs" mode="ClassExample"/>
		<xsl:apply-templates select="Members"/>
		#Document Change Here#
	</xsl:if>
</xsl:template>

<xsl:template match="Base">
	<xsl:if test="BaseTypeName">	
		<font face="Verdana" size="2">
		<b>Inherits From: <xsl:value-of select="BaseTypeName"/></b>
		<xsl:if test="ExcludedBaseTypeName">
			[<i>Note:</i> When implementing the <xsl:value-of select="ExcludedLibraryName"/> library, this type inherits from <xsl:value-of select="ExcludedBaseTypeName"/>.]
		</xsl:if>
		<br/><br/>
		</font>
	</xsl:if>
</xsl:template>

<xsl:template match="Members">
	<xsl:apply-templates select="Member">
             <!-- If there comes a MemberType that begins with a A or B, then
             with this mechanism, the constructors will not be listed first -->       
            <xsl:sort select="MemberType" order="ascending" />
            <xsl:sort select="@MemberName" order="ascending" />
        </xsl:apply-templates>
</xsl:template>

<xsl:template match="summary">
	<xsl:apply-templates/>
</xsl:template>


<xsl:template match="ThreadingSafetyStatement">
		<font face="Verdana" size="2">
				<b>Thread Safety: </b>
				<xsl:apply-templates/>
		</font><br/><br/>
</xsl:template>


<xsl:template match="remarks">

		<!-- Add boilerplate for types that are non-CLS compliant.-->
		<!-- repeated this code here, because need to create description block if doesn't already exist, to hold non-compliant info.-->
		<xsl:if test="not(ancestor::Member[1])">
			<xsl:if test="ancestor::Type[AssemblyInfo/Attributes/Attribute/AttributeName = 'CLSCompliantAttribute(false)'][1]">
				<font face="Verdana" size="2">
				<b>Description</b><br/>
				<p style="margin-left: 25">[<i>Note:</i> This type is not CLS-Compliant.]<br/><br/></p>
				</font>
			</xsl:if>
		</xsl:if>
		
		<p style="margin-left: 25"><xsl:apply-templates/></p>
</xsl:template>


<!-- This special-case adds a Description heading to remarks that 
contain something other than non-note blocks.-->
<!-- Joel Marcey, 24 June 2010, added text() condition -->
<xsl:template match="remarks[text() or para/text() or block[@type='note'] or para/block[@type='note']]">
		<font face="Verdana" size="2">
				<b>Description</b>
		<br/>
		<p style="margin-left: 25">
		
		<!-- Add boilerplate for types that are non-CLS compliant.-->
		<xsl:if test="not(ancestor::Member[1])">
			<xsl:if test="ancestor::Type[AssemblyInfo/Attributes/Attribute/AttributeName = 'CLSCompliantAttribute(false)'][1]">
				[<i>Note:</i> This type is not CLS-Compliant.]<br/><br/>
			</xsl:if>
		</xsl:if>
		
		<xsl:apply-templates/></p>
		</font>
</xsl:template>


<xsl:template match="TypeSignature">
	<!--<xsl:if test="@Language='C#'">-->
		<p style="margin-left: 25;background-color: #18E7E7">
			<!--<font face="Verdana" size="1">[C#]<br/></font>-->
			<font face="Verdana" size="1">[<xsl:value-of select="@Language"/>]<br/></font>
			<font face="Courier New" size="2"><b><xsl:value-of select="@Value"/></b></font>
		</p>
	<!--</xsl:if>-->
</xsl:template>

<xsl:template match="Member">
	<!-- do not want to print out if the member name is value__ ... these are in enums -->
	<xsl:if test="not(normalize-space(@MemberName)='value__')">
		<xsl:variable name="ebname" select="ExcludedLibrary"/>
		<p>
			#Page Break Here#
			<xsl:if test="Excluded='1'">
				<span style="background-color: #18F7D7">
					<u><b>The following member must be implemented if the <xsl:value-of select="$ebname"/> library is present in the implementation.</b></u>
				</span><br/>
			</xsl:if>
			<font face="Verdana" size="5" color="#A0A0A0">
			<b>
				<xsl:value-of select="ancestor::Type[1]/@Name"/>
				<xsl:if test="not(normalize-space(@MemberName)='.ctor')">
					<xsl:value-of select="'.'"/>
					<xsl:value-of select="@MemberName"/>
				</xsl:if>
				<xsl:if test="normalize-space(MemberType)='Method' or normalize-space(MemberType)='Constructor'">
					<xsl:apply-templates select="Parameters" mode="memberheading"/>
				</xsl:if>
				<xsl:value-of select="' '"/><xsl:value-of select="MemberType"/>
			</b>
			</font>
		</p>
		<xsl:apply-templates select="MemberSignature"/>
		<xsl:apply-templates select="Docs" mode="MemberSummary"/>
		<xsl:apply-templates select="Attributes"/>
		<xsl:apply-templates select="Interfaces"/>
		<xsl:apply-templates select="Parameters"/>

		<xsl:apply-templates select="ReturnValue"/>
		<xsl:apply-templates select="Docs" mode="MemberDescription"/>
		<xsl:apply-templates select="Docs" mode="MemberPermissions"/>	
			
	</xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="MemberSummary">
	<font face="Verdana" size="2">
		<b>Summary</b>
		<p style="margin-left: 25"><xsl:apply-templates select="summary"/></p>
	</font>
</xsl:template>

<xsl:template match="MemberSignature">
	<p style="margin-left: 25;background-color: #18E7E7">		
		<font face="Verdana" size="1">[<xsl:value-of select="@Language"/>]<br/></font>
		<font face="Courier New" size="2">
			<xsl:choose>
				<xsl:when test="@Language='C#'">
					<!-- Remove the ; at the end of the signature -->
					<!-- ...but need to be careful of properties -->
					<xsl:choose>
						<xsl:when test="contains(@Value, '}')">
						    <b><xsl:value-of select="@Value"/></b>
						</xsl:when>
						<xsl:when test="contains(@Value, ';')">	
						    <b><xsl:value-of select="substring-before(@Value,';')"/></b>
						</xsl:when>
						<xsl:otherwise>
				                    <b><xsl:value-of select="@Value"/></b>	
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
			            <b><xsl:value-of select="@Value"/></b>	
				</xsl:otherwise>
			</xsl:choose>
		</font>
	</p>
</xsl:template>

<xsl:template match="Docs" mode="ClassSummary">
	<xsl:if test="summary">
		<font face="Verdana" size="2">
			<b>Summary</b>
		</font><br/>
		<font face="Verdana" size="2">
			<p style="margin-left: 25"><xsl:apply-templates select="summary"/></p>
		</font>
	</xsl:if>
</xsl:template>

<!-- JM 21 Nov 2004 -->
<!-- Adding Returns given the fact that some of the new-->
<!-- generic delegates have return other than void-->
<xsl:template match="Docs" mode="ClassReturns">
     <xsl:if test="returns">
         <font face="Verdana" size="2">
             <b>Returns</b>
	 </font><br/><br/>
         <xsl:apply-templates select="returns"/>
     </xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="ClassParameters">
	<xsl:if test="param">
		<font face="Verdana" size="2">
			<b>Parameters</b>
		</font><br/><br/>

	 	<xsl:apply-templates select="following-sibling::Docs" mode="MemberParameters"/>
	
		<p style="margin-left: 25">
			<table border="1" width="100%">
  				<tr>
    				<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Parameter</b>
						</font>
					</td>
    				<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Description</b>
						</font>
					</td>
  				</tr>
				<xsl:apply-templates select="param"/>
			</table>
		</p><br/>
	</xsl:if>
</xsl:template>


<xsl:template match="Docs" mode="ClassExample">
    <xsl:apply-templates select="example"/>
</xsl:template>

<!-- JM 27 Dec 2004 -->
<!-- For Classes (sparked by MethodInfo)...just mimicked the MemberPermissions -->
<xsl:template match="Docs" mode="ClassPermissions">
    	<xsl:if test="permission">
		<font face="Verdana" size="2">
			<b>Permissions</b>
		</font><br/><br/>
		<p style="margin-left: 25">
			<table border="1" width="100%">
	  			<tr>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Permission</b>
						</font>
					</td>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Description</b>
						</font>
					</td>
	  			</tr>
				<xsl:for-each select="permission">
		  			<tr>
		    			<td>
							<font face="Verdana" size="2">
							<b><xsl:value-of select="local:crefBreak(string(@cref), 40)"/></b>
							</font>
						</td>
		    			<td>
							<font face="Verdana" size="2">
								<xsl:apply-templates/>
							</font>
						</td>
		  			</tr>
				</xsl:for-each>
			</table>
		</p><br/>
	</xsl:if>
</xsl:template>

<xsl:template match="Interfaces[*]">
	<font face="Verdana" size="2">
		<b>Implements: </b>
		<ul>
			<b><xsl:apply-templates/></b>
		</ul>
	</font>
</xsl:template>



<!--Note: the following code assumes each excluded interface have 
only one <ExcludedLibraryName> field.-->
<xsl:template match="Interface">
		<li>
			<xsl:value-of select="InterfaceName"/>
			<xsl:variable name="interface-excluded-status" select="Excluded"/>
			<xsl:if test="$interface-excluded-status='1'">			
				[<i>Note:</i> This interface requires the <xsl:value-of select="ExcludedLibraryName"/> library.]
			</xsl:if>
		</li>
</xsl:template>

<xsl:template match="AssemblyInfo">
	<font face="Verdana" size="2">
		<b>Assembly Info: </b>
		<ul>
			<li><i>Name: </i> <xsl:value-of select="AssemblyName"/></li>
			<li><i>Public Key: </i> <xsl:value-of select="AssemblyPublicKey"/></li>
			<li><i>Version: </i> <xsl:value-of select="AssemblyVersion"/></li>
			<li><i>Attributes:</i> <xsl:apply-templates select="Attributes"/></li>
		</ul>
	</font>
</xsl:template>

<xsl:template match="Attributes[*]">
		<font face="Verdana" size="2">
			<xsl:if test="not(ancestor::AssemblyInfo)">
				<b>Type Attributes: </b>
			</xsl:if>
			<ul>
				<xsl:apply-templates select="Attribute"/>
			</ul>
		</font>
</xsl:template>

<xsl:template match="Attribute">
	<li>
	<xsl:value-of select="AttributeName"/>
	<xsl:if test="Excluded='1'">
		[<i>Note:</i> This attribute requires the <xsl:value-of select="ExcludedLibraryName"/> library.]	
	</xsl:if>
	</li>
</xsl:template>

<xsl:template match="Docs" mode="ClassDescription">
	<xsl:if test="remarks">
		<xsl:apply-templates select="remarks"/>
	</xsl:if>

	<xsl:if test = "seealso">	
		<font face="Verdana" size="2">
			<b>See Also </b>
		</font>
		<ul>
			<xsl:apply-templates select="seealso"/>
		</ul>
	</xsl:if>
</xsl:template>

<xsl:template match="seealso">
	<xsl:if test="@cref">
		<li>
			<xsl:apply-templates select="@cref"/>
		</li>
	</xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="MemberDescription">
	<xsl:if test="remarks">
		<xsl:apply-templates select="remarks"/>
	</xsl:if>
	<!-- Tests to see if the exception node exists -->
	<!-- Not in its own template due to complications with trying to keep
	     all of the exceptions in the class within one template -->
	<xsl:if test="exception">
		<font face="Verdana" size="2">
			<b>Exceptions</b>
		</font><br/><br/>
		<p style="margin-left: 25">
			<table border="1" width="100%">
	  			<tr>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Exception</b>
						</font>
					</td>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Condition</b>
						</font>
					</td>
	  			</tr>
				<xsl:for-each select="exception">
		  			<tr>
		    			<td>
							<font face="Verdana" size="2">
							<b><xsl:value-of select="local:crefBreak(string(@cref), 40)"/></b>
							</font>
						</td>
		    			<td>
							<font face="Verdana" size="2">
								<xsl:apply-templates/>
							</font>
						</td>
		  			</tr>
				</xsl:for-each>
			</table>
		</p><br/>
	</xsl:if>
	<xsl:apply-templates select="example"/>
	<xsl:if test="seealso">
		<font face="Verdana" size="2">
			<b>See Also </b>
		</font>
		<ul>
			<xsl:apply-templates select="seealso"/>
		</ul>
	</xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="MemberPermissions">
	<xsl:if test="permission">
		<font face="Verdana" size="2">
			<b>Permissions</b>
		</font><br/><br/>
		<p style="margin-left: 25">
			<table border="1" width="100%">
	  			<tr>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Permission</b>
						</font>
					</td>
	    			<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Description</b>
						</font>
					</td>
	  			</tr>
				<xsl:for-each select="permission">
		  			<tr>
		    			<td>
							<font face="Verdana" size="2">
							<b><xsl:value-of select="local:crefBreak(string(@cref), 40)"/></b>
							</font>
						</td>
		    			<td>
							<font face="Verdana" size="2">
								<xsl:apply-templates/>
							</font>
						</td>
		  			</tr>
				</xsl:for-each>
			</table>
		</p><br/>
	</xsl:if>
</xsl:template>
	
<xsl:template match="c/para">
		<xsl:apply-templates/><br/><br/>
</xsl:template>	
	
<xsl:template match="para">
	<font face="Verdana" size="2"><xsl:apply-templates/><br/><br/></font>
</xsl:template>

<xsl:template match="para" mode="NoBreaks">
	<font face="Verdana" size="2"><xsl:apply-templates/></font>
</xsl:template>

<xsl:template match="returns">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="see">
	<xsl:choose>
		<xsl:when test="@langword">
			<!-- JM Nov 14 -->
                        <!-- Modified to fixed width font -->
			<font face="Courier" size="2">
				<xsl:value-of select="@langword"/>
			</font>
			<xsl:if test="@langword='Required Behavior'">
				<br/><br/>
			</xsl:if>
		</xsl:when>
		<xsl:when test="@cref">		
			<xsl:apply-templates select="@cref"/>
		</xsl:when>
	</xsl:choose>
</xsl:template>

<!-- JM 7 June 2004 -->
<!-- Changed to experiment with reducing the wordiness of displaying the entire qualified type/member -->
<!-- name within the rendition. Instead of System.Collections.Generic.IList<T>, just display IList<T> -->
<!-- Currently only doing within crefs that are found in primary text -->

<xsl:template match="@cref">
	<xsl:if test="ancestor::permission[1] or ancestor::exception[1] or ancestor::param[1]">
		<font face="Courier" size="2">
			<xsl:value-of select="local:crefBreak(string(.),50)"/>
		</font>
	</xsl:if>
	
	<xsl:if test="not(ancestor::permission[1]) and not(ancestor::exception[1]) and not(ancestor::param[1])">
		<font face="Courier" size="2">
			<xsl:value-of select="local:stripCref(string(.))"/>
		</font>
	</xsl:if>

</xsl:template>

<xsl:template match="paramref">
	<font face="Verdana" size="2">
		<i><xsl:value-of select="@name"/></i>
	</font>
</xsl:template>

<xsl:template match="item" mode="bullet">
	<ul>
		<xsl:apply-templates select="term" mode="bullet"/>
	</ul>
</xsl:template>

<xsl:template match="list">
	<xsl:choose>
		<xsl:when test="@type='bullet'">
			<xsl:apply-templates select="item" mode="bullet"/>
		</xsl:when>
		<xsl:when test="@type='table'">
				<table border="1" width="100%">
		  			<tr>
		    				<td bgcolor="#33CCCC">
							<font face="Verdana" size="2">
								<b><xsl:value-of select="./listheader/term"/></b>
							</font>
						</td>
						<xsl:for-each select="./listheader/description">
		    					<td bgcolor="#33CCCC">
								<font face="Verdana" size="2">
									<b><xsl:apply-templates/></b>
								</font>
							</td>
						</xsl:for-each>
		  			</tr>
					<xsl:for-each select="./item">
			  			<tr>
			    				<td>
								<font face="Verdana" size="2">
									<xsl:apply-templates select="term" mode="table"/>
								</font>
							</td>
							<xsl:for-each select="description">
			    					<td>
									<font face="Verdana" size="2">
										<xsl:apply-templates/>
									</font>
								</td>
							</xsl:for-each>
			  			</tr>
					</xsl:for-each>
				</table>
				<br/>
		</xsl:when>
		<xsl:when test="@type='number'">
		<font face="Verdana" size="2">
			<ol> 
			<xsl:for-each select="item/term">
				<li><xsl:apply-templates/></li>
			</xsl:for-each>
			</ol>
		</font>
		</xsl:when>
	</xsl:choose>
</xsl:template>

<xsl:template match="term" mode="bullet">
		<font face="Verdana" size="2">
			<li><xsl:apply-templates/></li>
		</font>
</xsl:template>

<xsl:template match="description">
	<xsl:apply-templates/>
</xsl:template>


<xsl:template match="term" mode="table">
		<font face="Verdana" size="2">
			<xsl:apply-templates/>
		</font>
</xsl:template>

<xsl:template match="ReturnValue">
	<xsl:choose>
		<xsl:when test="preceding-sibling::MemberType='Property'">
			<xsl:apply-templates select="following-sibling::Docs" mode="PropertyReturn"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="not(ReturnType='System.Void')">
				<xsl:apply-templates select="following-sibling::Docs" mode="MemberReturn"/>
			</xsl:if>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="Parameters" mode="memberheading">(<xsl:apply-templates select="Parameter" mode="memberheading"/>)</xsl:template>

<xsl:template match="Parameter" mode="memberheading">
		<xsl:value-of select="@Type"/>
		<!-- Don't print out the , if we are at the last parameter -->
		<xsl:if test="not(position()=last())">
			<xsl:value-of select="', '"/>
		</xsl:if>
</xsl:template>

<xsl:template match="Parameters">
	<xsl:apply-templates select="following-sibling::Docs" mode="MemberParameters"/>

</xsl:template>

<xsl:template match="Docs" mode="MemberParameters">
	<xsl:if test="param">
		<font face="Verdana" size="2">
			<b>Parameters</b>
		</font><br/><br/>


		<p style="margin-left: 25">
			<table border="1" width="100%">
  				<tr>
    				<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Parameter</b>
						</font>
					</td>
    				<td bgcolor="#33CCCC">
						<font face="Verdana" size="2">
							<b>Description</b>
						</font>
					</td>
  				</tr>
				<xsl:apply-templates select="param"/>
			</table>
		</p><br/>
	</xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="MemberReturn">
	<xsl:if test="returns">
		<font face="Verdana" size="2">
			<b>Return Value</b>
		</font><br/>
		<p style="margin-left: 25"><xsl:apply-templates select="returns"/></p>
	</xsl:if>
</xsl:template>

<xsl:template match="Docs" mode="PropertyReturn">
	<xsl:if test="value">
		<font face="Verdana" size="2">
			<b>Property Value</b>
		</font><br/>
		<p style="margin-left: 25"><xsl:apply-templates select="value"/></p>
	</xsl:if>
</xsl:template>

<xsl:template match="code">
<xsl:if test="@lang">
<font face="Verdana" size="1">[<xsl:value-of select="@lang"/>]<br/></font>
</xsl:if>
	<font face="Courier New" size="2">
			<pre>
				<xsl:value-of select="current()"/>
			</pre>
	</font>
</xsl:template>

<xsl:template match="c">
<xsl:if test="@lang">
	<font face="Verdana" size="1">[<xsl:value-of select="@lang"/>]<br/></font>
</xsl:if>
	<font face="Courier New" size="2">
		<xsl:choose>
			<xsl:when test = "para">
				<xsl:for-each select="para">
					<xsl:apply-templates select="."/><br/>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates/>
			</xsl:otherwise>
		</xsl:choose>
	</font>
</xsl:template>

<xsl:template match="SPAN">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="block[@type='note']">
	<font face="Verdana" size="2">[<i>Note:</i><xsl:value-of select="' '"/></font>
        <!-- JM 15 November 2004 -->
        <!-- IGNORE 12 Nov 2004 UPDATE! COMMENTED OUT -->
        <!-- TRYING TO SIMPLIFY Notes -->
        <!-- stemmed from needing a table inside of note and doing a "when" wouldn't get us there -->
        <!-- needed the table in a note in System.Decimal description -->
        <!--<xsl:choose>-->
            <!--<xsl:when test="para">-->
                <!-- JM 12 November 2004 -->
                <!-- Updated in support of no paragraph breaks in notes as indicated by-->
                <!-- edition 1 SecurityElement class in the class description-->
                <!-- Not too sure why NoBreaks was used for notes in the first place??-->
                <!--<xsl:apply-templates select="para" mode="NoBreaks"/>-->
                <!--<xsl:apply-templates select="para"/>-->
	    <!--</xsl:when>-->
            <!--<xsl:otherwise>-->
                <!--<xsl:apply-templates/>-->
            <!--</xsl:otherwise>-->
        <!--</xsl:choose>-->
        <xsl:apply-templates/>	
        <font face="Verdana" size="2">]</font>
	<!--<xsl:if test="list">-->
	    <br/><br/>
	<!--</xsl:if>-->
</xsl:template>

<xsl:template match="block">
	<xsl:choose>
		<xsl:when test="@type='usage'">
			<h5 style="font-family:Verdana;margin-left:-0px">Usage</h5>
		</xsl:when>
		<xsl:when test="@type='overrides'">
			<h5 style="font-family:Verdana;margin-left:-0px">How and When to Override</h5>
		</xsl:when>
		<xsl:when test="@type='behaviors'">
			<h5 style="font-family:Verdana;margin-left:-0px">Behaviors</h5>
		</xsl:when>
		<xsl:when test="@type='default'">
			<h5 style="font-family:Verdana;margin-left:-0px">Default</h5>
		</xsl:when>
		<xsl:otherwise>
			<h5 style="font-family:Verdana;margin-left:-0px"><xsl:value-of select="@type"/></h5>
		</xsl:otherwise>
	</xsl:choose>
	<p style="margin-left: 25"><xsl:apply-templates/></p>
</xsl:template>

<!-- Will this behave like exception (i.e. uplevel into Docs (MemberDescription mode)?-->
<xsl:template match="example">
<font face="Verdana" size="2">
		<b>Example</b>
</font><br/>
<p style="margin-left: 25"><xsl:apply-templates/></p>
</xsl:template>

<xsl:template match="sup">
	<sup><xsl:apply-templates/></sup>
</xsl:template>

<xsl:template match="sub">
	<sub><xsl:apply-templates/></sub>
</xsl:template>

<!-- JM 21 November 2004 -->
<!-- Further super and sub script support -->

<xsl:template match="superscript">
	<sup><xsl:value-of select="@term"/></sup>
</xsl:template>

<xsl:template match="subscript">
	<sub><xsl:value-of select="@term"/></sub>
</xsl:template>

<xsl:template match="entity">
	<xsl:value-of disable-output-escaping="yes" select="."/>
</xsl:template>

<!-- JM 24 Nov 2004 -->
<!-- Changed escape values to hex ** DIDN't WORK -->
<!-- THEN just used symbol font  -->
<!-- and other HACKS (permille and onequarter are hacks -->

<!--escaping entity refs -->
<xsl:template match="pi">
	<!--<xsl:text disable-output-escaping="yes">&amp;#928;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x03C0;</xsl:text>-->
	<font face="Symbol">&#112;</font>
</xsl:template>

<xsl:template match="theta">
	<!--<xsl:text disable-output-escaping="yes">&amp;#920;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x03B8;</xsl:text>-->
    <font face="Symbol">&#113;</font>
</xsl:template>

<xsl:template match="onequarter">
	<!--<xsl:text disable-output-escaping="yes">&amp;#188;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x00BC;</xsl:text>-->
    <sup>1</sup>/<sub>4</sub>
</xsl:template>

<xsl:template match="permille">
	<!--<xsl:text disable-output-escaping="yes">&amp;#8240;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x2030;</xsl:text>-->
    <!--<p>&permil;</p>-->
	<sup>o</sup>/<sub>oo</sub>
</xsl:template>

<!-- JM 16 Dec 2004 -->
<!-- Couldn't get it with symbol fonts or unicode chars, so used images -->
<xsl:template match="geq">
	<!--<xsl:text disable-output-escaping="yes">&amp;#8805;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x2265;</xsl:text>-->
    <font face="Symbol">&#179;</font>
	<!--<img src="geq.gif"/>-->
</xsl:template>

<xsl:template match="leq">
	<!--<xsl:text disable-output-escaping="yes">&#8804;</xsl:text>-->
    <!--<xsl:text disable-output-escaping="yes">&amp;#x2264;</xsl:text>-->
    <font face="Symbol">&#163;</font>
    <!--<img src="leq.gif"/>-->
	<!--<p style="font-size:medium;">&le;</p>-->
</xsl:template>

<xsl:template match="plusminus">
	<font face="Symbol">&#177;</font>
</xsl:template>

<xsl:template match="infinity">
   <!--<font face="Symbol">&#165;</font>-->
   <!--<img src="inf.gif" alt="infinity" />-->
   <font face="Symbol">&#8734;</font>
</xsl:template>

<xsl:template match="whitespace">
	<xsl:text> </xsl:text>		
</xsl:template>
	
<xsl:template match="param">

	<tr>
		<td>
			<font face="Verdana" size="2">
				<i><xsl:value-of select="@name"/></i>
			</font>
		</td>
		<td>
			<font face="Verdana" size="2">
				<xsl:apply-templates/>
			</font>
		</td>
	</tr>

</xsl:template>


<msxsl:script language="JScript" implements-prefix="local"><![CDATA[

	/* --------------------------------------------------
		Name: crefBreak
		
		Parameters:
			text - the cref text to break up
			maxlength - the maximum column width allowed in a cref
		
		Description: 
			This function adds spaces to cref values so they will wrap 
			in word tables instead of scrolling off to the right.  Makes
			them printable.  It also strips the extraneous characters.
	  ---------------------------------------------------- */
	function crefBreak(text, maxlength){
		text = stripCref(text);
		return columnBreak(text, maxlength);
	}

	/* --------------------------------------------------
		Name: stripCref
		
		Parameters:
			text - the cref text to strip of extraneous characters
		
		Description: 
			This function strips everything to the left
			of (and including) the colon if it appears in the cref. next, we strip 
			everything to the right of (and including) a left paren char if
			it appears. something like M:Type.ThisType(Int32) therefore
			will render as Type.ThisType (the appropriate fmt for crefs)
	  ---------------------------------------------------- */
	function stripCref(text)
	{
		var iColon = text.indexOf(":") + 1;
		var iParen = text.indexOf("(");
		
		if (iParen == -1)
			iParen = text.length;
			
		return text.substring(iColon,iParen);		
	}
	
	/* --------------------------------------------------
		Name: columnBreak
		
		Parameters:
			text - the text to break up
			maxlength - the maximum column width allowed in a cref
		
		Description: 
			This function adds spaces to text strings so they will wrap 
			in word tables instead of scrolling off to the right.  Makes
			them printable.
	  ---------------------------------------------------- */
    function columnBreak(text, maxlength) {
		var output = null;
		var lastSpace = -1;
		var lastDot = -1;
		var currentString = 0;

		for(var i=0;i<text.length;i++)
		{
			if(text.charAt(i) == " ") 
			{
				lastSpace = i;
				currentString = 0;
			}else{
				currentString++;
			}
			
			if(text.charAt(i) == ".") 
			{
				lastDot = i;
			}
			
			if(currentString > maxlength)
			{
				if(lastDot > lastSpace && lastDot > -1)
				{
					text =text.substring(0,lastDot+1)+" "+text.substring(lastDot+1,text.length);
					i=lastDot;
					lastDot = -1;
				}
				
				//What to do if there is no dot to break at?  Leave it alone?
			}
			
		}
		
      return text;
    }
    
    function findLastDot(text)
    {
		var lastDot = -1;0;

		for(var i=0;i<text.length;i++)
		{
			if(text.charAt(i) == ".") 
			{
				lastDot = i;
			}
		}
		if(lastDot > -1)
		{
			    text =text.substring(lastDot+1, text.Length)
	    }
      return text;
    }
    
    function findSecondToLastDot(text)
    {
        var output = null;
		var lastSpace = -1;
		var secondToLastDot = -1
		var lastDot = -1;
		var currentString = 0;

		for(var i=0;i<text.length;i++)
		{
			if(text.charAt(i) == ".") 
			{
			    secondToLastDot = lastDot;
				lastDot = i;
			}
		}
		if(lastDot > -1)
		{
			    text =text.substring(secondToLastDot+1, text.Length)
	    }
      return text;
    }
]]>
</msxsl:script>

</xsl:stylesheet>
<!-- Stylus Studio meta-information - (c)1998-2001 eXcelon Corp.
<metaInformation>
<scenarios/>
</metaInformation>
-->
